import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.*;
import javax.imageio.*;
import javax.swing.border.*;
import java.awt.event.ActionListener;
import java.net.*;
import java.io.*;


public class RegionLabelFrame extends JFrame implements 
     ActionListener, ListSelectionListener
{
    private RegionLabelData mdata = null; // model data;
    private SegmentImagePanel segmentImage = null; // segmented image;
    private boolean isDataChanged = false;  // is the model data changed?

   private String currentLabel=""; //label for the active segment; 

    private Vector newLabelList = new Vector(); // keep the recently typed;

    private File imageFileDirectory = new File(".");
    private File segmentFileDirectory = new File(".");
    private File outputFileDirectory = new File(".");
    private File templateFileDirectory = new File("./template");
    private File outputFile = null;  //output file

    private JLabel currentIndexLabel = null;
    private JComboBox labelListCombo = null;
   // private JTextField labelTextField = null;
    private WordTextField labelTextField = null;
    private JButton forwardButton = null;
    private JButton backwardButton = null;
    private JButton clearListButton = null;

    //file filter 
    private ExtensionFileFilter imageFileFilter = null;
    private ExtensionFileFilter segmentFileFilter = null;
    private ExtensionFileFilter outputFileFilter = null;
    private ExtensionFileFilter templateFileFilter = null;
    
    //Menu 
    private JMenu fileMenu = null;
    private JMenu viewMenu = null;
    private JMenu optionMenu = null;
    private JMenu helpMenu = null;

   //display wordnet page
    private JFrame wordNetFrame;
    private JTextPane wordNetPane;

    //display corel words;
    private JList corelWordList;
    RegionLabelFrame()
    {
        setSize(new Dimension(600, 500));
        attach_menu();
       // regionLabelPanel = new RegionLabelPanel();
        JPanel contentPane = (JPanel)getContentPane();
        contentPane.setLayout(new BorderLayout());
        segmentImage = new SegmentImagePanel();
        segmentImage.addActionListener(this);
        segmentImage.setPreferredSize(new Dimension(500,500));
        segmentImage.setBorder(BorderFactory.createLoweredBevelBorder());
        segmentImage.setMouseClickedNotified(true);
        contentPane.add(segmentImage, BorderLayout.CENTER); 
        
       // add(new JSeparator(JSeparator.HORIZONTAL));

        JPanel labelPanel = createLabelPanel();
        contentPane.add(labelPanel, BorderLayout.SOUTH);
    
        // create a window to display wordnet page
        createWordNetFrame();
 
        // set the initial status for all kinds of component such as menu items and buttons; 
        setBeforeLoadComponentStatus();
 
        createFileFilter();
    }

    /* Component status before a file is loaded */
    private void  setBeforeLoadComponentStatus()
    {
        // set initial status for the menu items;
        setMenuItemStatus(fileMenu, false, RegionLabelConstants.FILE_SAVE_INDEX);    
        setMenuItemStatus(fileMenu, false, RegionLabelConstants.FILE_SAVE_AS_INDEX);    
        setMenuItemStatus(fileMenu, false, RegionLabelConstants.FILE_CLOSE_INDEX);    
        setMenuItemStatus(fileMenu, false, RegionLabelConstants.FILE_LOAD_TEMPLATE_INDEX);    
        setMenuItemStatus(fileMenu, false, RegionLabelConstants.FILE_SAVE_TEMPLATE_INDEX);    
       
        // set initial status for all other components;
        forwardButton.setEnabled(false); 
        backwardButton.setEnabled(false); 
        clearListButton.setEnabled(false); 
        labelTextField.setEnabled(false); 
        labelListCombo.setEnabled(false); 
    }
    
    /* Component status after a file is loaded */
    private void  setAfterLoadComponentStatus()
    {
        // set initial status for the menu items;
        setMenuItemStatus(fileMenu, true, RegionLabelConstants.FILE_SAVE_INDEX);    
        setMenuItemStatus(fileMenu, true, RegionLabelConstants.FILE_SAVE_AS_INDEX);    
        setMenuItemStatus(fileMenu, true, RegionLabelConstants.FILE_CLOSE_INDEX);    
        setMenuItemStatus(fileMenu, true, RegionLabelConstants.FILE_LOAD_TEMPLATE_INDEX);    
        setMenuItemStatus(fileMenu, true, RegionLabelConstants.FILE_SAVE_TEMPLATE_INDEX);    
      
        // set initial status for all other components;
        forwardButton.setEnabled(true); 
        backwardButton.setEnabled(true); 
        clearListButton.setEnabled(true); 
        labelTextField.setEnabled(true); 
        labelListCombo.setEnabled(true); 
    }
   
    private void setMenuItemStatus(JMenu menu, boolean b, int itemIndex)
    {
        JMenuItem item = menu.getItem(itemIndex);    
        if(item != null)
        {
            item.setEnabled(b);    
        }
    }

    private void createFileFilter()
    {
        imageFileFilter = new ExtensionFileFilter();
        imageFileFilter.addExtension("jpg");
        imageFileFilter.addExtension("gif");
        //add more image formats here
        //.......

        segmentFileFilter = new ExtensionFileFilter();
        segmentFileFilter.addExtension("seg");
        outputFileFilter = new ExtensionFileFilter();
        outputFileFilter.addExtension("lab");
        templateFileFilter = new ExtensionFileFilter();
        templateFileFilter.addExtension("txt");
        templateFileFilter.addExtension("lab");
    }

    public JMenuBar attach_menu()
    {
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        // File menu
        fileMenu = new JMenu(RegionLabelConstants.FILE_MENU);
        menuBar.add(fileMenu);
        JMenuItem newFileItem = fileMenu.add(RegionLabelConstants.FILE_NEW_ITEM);
        newFileItem.addActionListener(this);
        JMenuItem openFileItem = fileMenu.add(RegionLabelConstants.FILE_OPEN_ITEM);
        openFileItem.addActionListener(this);
        JMenuItem saveFileItem = fileMenu.add(RegionLabelConstants.FILE_SAVE_ITEM);
        saveFileItem.addActionListener(this);
        JMenuItem saveAsFileItem = fileMenu.add(RegionLabelConstants.FILE_SAVE_AS_ITEM);
        saveAsFileItem.addActionListener(this);
        JMenuItem closeFileItem = fileMenu.add(RegionLabelConstants.FILE_CLOSE_ITEM);
        closeFileItem.addActionListener(this);
        fileMenu.addSeparator();
        JMenuItem loadTemplateItem = fileMenu.add(RegionLabelConstants.FILE_LOAD_TEMPLATE_ITEM);
        loadTemplateItem.addActionListener(this);
        JMenuItem saveTemplateItem = fileMenu.add(RegionLabelConstants.FILE_SAVE_TEMPLATE_ITEM);
        saveTemplateItem.addActionListener(this);
        fileMenu.addSeparator();
        JMenuItem quitItem = fileMenu.add(RegionLabelConstants.FILE_QUIT_ITEM);
        quitItem.addActionListener(this);

        //View menu
        viewMenu = new JMenu(RegionLabelConstants.VIEW_MENU);
        menuBar.add(viewMenu);
        /*viewMenu.add("Zoom in");
        viewMenu.add("Zoom out");
        viewMenu.addSeparator();
        viewMenu.add("Hide All segments");
        viewMenu.add("Hide Non-active segments");*/

        //Option menu
        optionMenu = new JMenu(RegionLabelConstants.OPTION_MENU);
        menuBar.add(optionMenu);
        JMenuItem changeTemplateItem = optionMenu.add(RegionLabelConstants.OPTION_CHANGE_TEMPLATEDIR_ITEM);
        optionMenu.add(changeTemplateItem);
        changeTemplateItem.addActionListener(this);
        
        //Help menu
        helpMenu = new JMenu(RegionLabelConstants.HELP_MENU);
        menuBar.add(helpMenu);
        JMenuItem aboutItem = helpMenu.add(RegionLabelConstants.HELP_ABOUT_ITEM);
        helpMenu.add(aboutItem);
        aboutItem.addActionListener(this);

        return menuBar;
    } 

    public JPanel createLabelPanel()
    {
        JPanel labelPanel = new JPanel();
        labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));
       // labelPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
      //  panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
       // panel1.setAlignmentX(Component.RIGHT_ALIGNMENT);
        currentIndexLabel = new JLabel("<html> <font size= 4> Current Segment:  </font></html>");
        panel1.add(currentIndexLabel);
        currentIndexLabel = new JLabel();
        currentIndexLabel.setPreferredSize(new Dimension(70,30));
        updateCurrentLabelIndex(0, 0);
        currentIndexLabel.setBorder(BorderFactory.createLineBorder(Color.black));
        panel1.add(currentIndexLabel);

        ImageIcon backwardButtonIcon = new ImageIcon("image/left.gif");
        ImageIcon forwardButtonIcon = new ImageIcon("image/right.gif");
        forwardButton = new JButton(forwardButtonIcon); 
        forwardButton.addActionListener(this);
        backwardButton = new JButton(backwardButtonIcon); 
        backwardButton.addActionListener(this);
        panel1.add(backwardButton);
        panel1.add(forwardButton);
        
        labelPanel.add(panel1);
        labelPanel.add(new JSeparator(JSeparator.HORIZONTAL));

        JPanel panel2 = new JPanel();
        panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
        panel2.add(Box.createRigidArea(new Dimension(0,5)));
        JLabel label = new JLabel("<html> <font size=4> Type a label or pick one from the lists below </font></html>");
        label.setAlignmentX(Component.RIGHT_ALIGNMENT);
        label.setAlignmentY(Component.TOP_ALIGNMENT);
        panel2.add(label);
        panel2.add(Box.createRigidArea(new Dimension(0,5)));
        labelTextField = new WordTextField();
        labelTextField.setForeground(Color.blue);
        Font f = labelTextField.getFont();
        labelTextField.setFont(new Font(f.getFontName(), f.getStyle(), 14));
        labelTextField.setPreferredSize(new Dimension(300, 30));
        labelTextField.setMouseOnWordNotified(true);
        labelTextField.addActionListener(this);
        labelTextField.setAlignmentX(Component.RIGHT_ALIGNMENT);
       // labelTextField.setAlignmentY(Component.TOP_ALIGNMENT);
        panel2.add(labelTextField);
        labelPanel.add(panel2);
        panel2.add(Box.createRigidArea(new Dimension(0,5)));
        labelPanel.add(new JSeparator(JSeparator.HORIZONTAL));

        JPanel panel3 = new JPanel();
         
        panel3.setLayout(new FlowLayout(FlowLayout.LEADING));
       ((FlowLayout)panel3.getLayout()).setHgap(20);
        // create the corel key words list;
        JPanel panel3_1 = new JPanel();
        panel3_1.setLayout(new BoxLayout(panel3_1, BoxLayout.Y_AXIS));
        panel3_1.setAlignmentY(Component.TOP_ALIGNMENT);
        label = new JLabel("<html><font size=4>Corel Key Words</font></html>");
        label.setAlignmentX(Component.RIGHT_ALIGNMENT);
        label.setAlignmentY(Component.TOP_ALIGNMENT);
        panel3_1.add(label);
        panel3_1.add(Box.createRigidArea(new Dimension(0,5)));
        corelWordList = new JList();
        f = corelWordList.getFont();
        corelWordList.setFont(new Font(f.getFontName(), f.getStyle(), 14));
        corelWordList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        corelWordList.setLayoutOrientation(JList.VERTICAL);
	corelWordList.setVisibleRowCount(-1);
	corelWordList.setBorder(BorderFactory.createLoweredBevelBorder());
    	corelWordList.addListSelectionListener(this);
	corelWordList.setSelectedIndex(-1);
        JScrollPane js = new JScrollPane(corelWordList);
        js.setPreferredSize(new Dimension(280, 100));
        panel3_1.add(js);

        JPanel panel3_2 = new JPanel();
        panel3_2.setLayout(new BoxLayout(panel3_2, BoxLayout.Y_AXIS));
        label = new JLabel("<html><font size=4>Template Word List</font><html>");
        label.setAlignmentX(Component.LEFT_ALIGNMENT);
        label.setAlignmentY(Component.TOP_ALIGNMENT);
        panel3_2.add(label);
        panel3_2.add(Box.createRigidArea(new Dimension(0,5)));
        
        labelListCombo = new JComboBox();
        labelListCombo.setPreferredSize(new Dimension(350, 25));
        //labelListCombo.setEditable(true);
        labelListCombo.addActionListener(this);
        f = labelListCombo.getFont();
        labelListCombo.setFont(new Font(f.getFontName(), f.getStyle(), 14));

        labelListCombo.setAlignmentX(Component.LEFT_ALIGNMENT);
        labelListCombo.setAlignmentY(Component.TOP_ALIGNMENT);
        panel3_2.add(labelListCombo);
        panel3_2.add(Box.createRigidArea(new Dimension(0,5)));
        clearListButton = new JButton("Clear List");
        clearListButton.addActionListener(this);
        clearListButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        clearListButton.setAlignmentY(Component.TOP_ALIGNMENT);
        panel3_2.add(clearListButton);
        panel3_2.add(Box.createRigidArea(new Dimension(0,40)));

         panel3.add(panel3_2);
         panel3.add(new JSeparator(JSeparator.VERTICAL));
         panel3.add(panel3_1);
 
        labelPanel.add(panel3);

        return labelPanel;
    }

    public JPanel createImagePanel()
    {
        JPanel imagePanel = new JPanel();
        //imagePanel.setPreferredSize(new Dimension(600,400));
        Border raisedetched = BorderFactory.createEtchedBorder(EtchedBorder.RAISED);
        imagePanel.setBorder(raisedetched);

        segmentImage = new SegmentImagePanel();
        imagePanel.add(segmentImage); 
        return imagePanel;
    }

    public void createWordNetFrame()
    {
        wordNetPane = new JTextPane();
        wordNetPane.setEditable(false);  // do not edit it
        JScrollPane js = new JScrollPane(wordNetPane);
       // js.getViewport().add(wordNetPane);
        wordNetFrame = new JFrame();
        wordNetFrame.getContentPane().add(js);
        wordNetFrame.setPreferredSize(new Dimension(400, 600));
        wordNetFrame.pack();
        wordNetFrame.setVisible(false);
        
    }

    public void setOutputFile(File file)
    {
        if(file != (File)null) {
            outputFile = new File(file.getPath());
        }
        else
        {
            outputFile = null;
        }
    }

    public void setOutputFile(String filename)
    {
        if(filename != (String)null) {
            outputFile = new File(filename);
        }
        else
        {
            outputFile = null;
        }
    }

    public void setDataChanged(boolean b)
    {
        isDataChanged = b;
    }

    public int getActiveSegmentIndex()
    {
         if(segmentImage != null)
         {
             return segmentImage.getActiveSegmentIndex();
         }

         return -1;
    }

    /* simply assume that 
          1) the segment file is in the same directory as the image file;
          2) the segment file differs from the image file only in the extension. 
          3) the segment file has an extension 'seg'. */
    public File getFileWithDifferentExtension(String fileName, String extension)
    {
        if(fileName == null)
            return null;

        int lastDotIndex = fileName.lastIndexOf('.');
        if(lastDotIndex < 0)
            return null;

        String name = fileName.substring(0, lastDotIndex+1);
        return new File(name + extension);
    }

    /* find the output file name for a new file.
       The default name is the same as the image name (except the extension).
       If the default name exists, then an index starting from 1 is appended.
       The index was incremented until the file is not available in the current
       directory. */
    public File getNameForNewFile(File imageFile)
    {
        File newFile = null;
        String name = imageFile.getName();
        String parent = imageFile.getParent();
        int lastDotIndex = name.lastIndexOf('.');
        if(lastDotIndex < 0)
            return null;

        String subname = name.substring(0, lastDotIndex); // remove the extension;

        int index = 0;
        while(true)
        {
            String s = "";
            if(index == 0)
                s = subname + ".lab";
            else
                s = subname + "_" + index + ".lab";

            newFile = new File(parent, s);
            if(!newFile.exists())
                break;
            
            index++;
        } 

        return newFile;
    }

    /* ListSelectionListener */
   /* public void itemStateChanged(ItemEvent e)
    {
       System.out.println(e.getStateChange());
       if(e.getStateChange() != ItemEvent.SELECTED)
           return;

       Object ob = e.getSource();
       if(ob instanceof JComboBox)
       {
          executeUpdateLabel((JComboBox)ob); 
       }
    }*/

    /* ListSelectionListener */
    public void valueChanged(ListSelectionEvent e)
    {
        if (e.getValueIsAdjusting() == 	true) {
	    String selectedStr  = (String)corelWordList.getSelectedValue();
            if(selectedStr != null)
                 executeUpdateLabel(selectedStr);
        }
    }

    /* Action Listener */
    public void actionPerformed(ActionEvent ev)
    {
        Object ob = ev.getSource();
        if(ob instanceof JMenuItem) executeMenuItem((JMenuItem)ob);
        else if(ob instanceof JButton)
        {
           JButton button = (JButton)ob;
           if(button == clearListButton)
               executeClearLabelList();
           else
                executeRotateSegment(button);
        }
        else if(ob instanceof JComboBox) executeUpdateLabel((JComboBox)ob);
        else if(ob instanceof JTextField) 
        {
            if(ev.getActionCommand().equals(WordTextField.MOUSE_MOVE_ON_WORD))
                executeDisplayWordNetPage((WordTextField)ob);
            else if(ev.getActionCommand().equals(WordTextField.MOUSE_MOVE_OFF_WORD))
                executeHideWordNetPage();
            else
                executeAssignLabel((WordTextField)ob);
        }
        else if(ob instanceof SegmentImagePanel)executeUpdateLabel((SegmentImagePanel)ob);
    }
    
    /************************************************************/
    /* This routine update the model view due to the change of the model data. 
       it should be called when the model data changes */
    private void updateRegionLabelView()
    {
        int index = getActiveSegmentIndex();
        updateCurrentLabelIndex(index, segmentImage.getNumSegments());

        String label = mdata.getSegmentLabelAt(index);
        labelTextField.setText(label);
        labelTextField.selectAll();
        labelTextField.requestFocusInWindow();
        currentLabel = label;  //update the current label;
    }
    
    private void updateCurrentLabelIndex(int index, int num)
    {
        if(num > 0) index++;

        String s1="";
        String s2="";
        if(index < 10)
            s1 = "00" + index ;
        else if(index < 100)
            s1 = "0" + index;
        else
           s1 = "" + index;
        
        if(num < 10)
            s2 = "00" + num ;
        else if(index < 100)
            s2 = "0" + num;
        else
           s2 = "" + num;


        String str = "<html> <font size = 4> <font color = #3300FF >"  + s1 + "</font color>" +
                     "<font color = #000000>/" + s2 + "</font color></font></html>";
        currentIndexLabel.setText(str); 
    }

    private void executeMenuItem(JMenuItem menuItem)
    {
        String menuText = menuItem.getText();
        if(menuText.compareTo(RegionLabelConstants.FILE_NEW_ITEM) == 0)
            executeNewLabelImage();
        else if(menuText.compareTo(RegionLabelConstants.FILE_OPEN_ITEM) == 0)
            executeOpenLabelImage();
        else if(menuText.compareTo(RegionLabelConstants.FILE_SAVE_ITEM) == 0)
            executeSaveFile();
        else if(menuText.compareTo(RegionLabelConstants.FILE_SAVE_AS_ITEM) == 0)
            executeSaveAsFile();
        else if(menuText.compareTo(RegionLabelConstants.FILE_CLOSE_ITEM) == 0)
            executeCloseFile(true);
        else if(menuText.compareTo(RegionLabelConstants.FILE_LOAD_TEMPLATE_ITEM) == 0)
            executeLoadTemplateFile();  
        else if(menuText.compareTo(RegionLabelConstants.FILE_SAVE_TEMPLATE_ITEM) == 0)
            executeSaveTemplateFile();  
        else if(menuText.compareTo(RegionLabelConstants.FILE_QUIT_ITEM) == 0)
               executeQuit();
        /*else if(menuText.compareTo(RegionLabelConstants.OPTION_HIDE_ALL_ITEM) == 0)
        else if(menuText.compareTo(RegionLabelConstants.OPTION_SHOW_ALL_ITEM) == 0)
        else if(menuText.compareTo(RegionLabelConstants.OPTION_SHOWALL_ITEM) == 0)*/
        else if(menuText.compareTo(RegionLabelConstants.OPTION_CHANGE_TEMPLATEDIR_ITEM) == 0)
            executeChangeTemplateDir();
        else if(menuText.compareTo(RegionLabelConstants.HELP_ABOUT_ITEM) == 0)
               executeAbout();
    }

/**************************START OF ROUNTINES FOR OPERATIONS ON 'FILE' MENU**************************/

    /***************************************************/
    /*This routine open a new image for labeling. It assumes that the image to be
    labeled has the same file name as the segment file and searches the segment file
    in the current directory. If the segment file is not found, it prompts the user
    to specify where the segment file is. 
    Note: Currently it doesn't check whether the image is labelled or not. Thus, 
    there is a risk of being overwritten if the image is already labelled.
    */
    private boolean executeNewLabelImage()
    {
        File imageFile = null;
        File segmentFile = null;
        
       //check if anything need to be saved before open a new file
        if(!executeCloseFile(false)) {
            return false;
        }

        JFileChooser fc = new JFileChooser();
        fc.setDialogType(JFileChooser.OPEN_DIALOG);
        fc.setFileFilter(imageFileFilter);
        fc.setDialogTitle(RegionLabelConstants.FILE_NEW_ITEM);
        fc.setCurrentDirectory(imageFileDirectory);
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) 
        {
            imageFile = fc.getSelectedFile();
            imageFileDirectory = imageFile.getParentFile(); //keep the directory;
            segmentFile = getFileWithDifferentExtension(imageFile.getPath(), "seg");
            if(!segmentFile.exists())
            {
               fc.setFileFilter(segmentFileFilter);
               fc.setDialogTitle("Open Segmentation File...");
               fc.setCurrentDirectory(segmentFileDirectory);
               System.out.println(segmentFile.getName());
               fc.setSelectedFile(new File(segmentFile.getName()));
                returnVal = fc.showOpenDialog(this);
                if(returnVal == JFileChooser.APPROVE_OPTION)
                {
                    segmentFile = fc.getSelectedFile();
                    segmentFileDirectory = segmentFile.getParentFile(); //keep the directory;
                }
                else
                {
              //      JOptionPane.showMessageDialog(this, 
               //             null, RegionLabelConstants.SEGMENT_FILE_ERROR, JOptionPane.ERROR_MESSAGE);
                    return false;
                }
            }

            /*create a new data model */
            mdata = new RegionLabelData(imageFile.getPath(), segmentFile.getPath());

            /* load the image and its segments */
            boolean b = loadSegmentImage(null);
	    if(b) // initialize the labels as 'unknown'
            {
                mdata.initializeLabels(segmentImage.getNumSegments());
            }
            else
            {
                //throws out error;
                return false;
            }

            setDataChanged(true);  // new file;
            File file = getNameForNewFile(imageFile);
            setOutputFile(file);
            this.setTitle(outputFile.getPath());

            // load corel key words;
            loadCorelKeyWords(imageFile.getName());           

            setAfterLoadComponentStatus();
            updateRegionLabelView();
            return true;
         }

         return false;
    }

    private boolean executeOpenLabelImage()
    {
        File labelFile = null;

        //If a file is currently being edited, ask if a save is needed.
        if(outputFile != null && !executeCloseFile(false)) {
            return false;
        }

        JFileChooser fc = new JFileChooser();
        fc.setDialogType(JFileChooser.OPEN_DIALOG);
        fc.setFileFilter(outputFileFilter);
        fc.setDialogTitle(RegionLabelConstants.FILE_OPEN_ITEM);
        fc.setCurrentDirectory(outputFileDirectory);
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) 
        {
            labelFile = fc.getSelectedFile();
            RegionLabelData data = RegionLabelData.readRegionLabelDataFromText(labelFile.getPath());
            if(data == null)
            {
                    JOptionPane.showMessageDialog(this, 
                            RegionLabelConstants.READ_FILE_ERROR, null, JOptionPane.ERROR_MESSAGE);
                    return false;
            } 
            
            outputFileDirectory = labelFile.getParentFile();

            mdata = data;
            if(!loadSegmentImage(labelFile.getParentFile()))
            {
                // throws out error;
                return false;
            }

            setDataChanged(false); // data has not changed yet;
            setOutputFile(labelFile); // 
            this.setTitle(outputFile.getPath());

            // load corel key words;
            loadCorelKeyWords(labelFile.getName());           

            setAfterLoadComponentStatus();
            updateRegionLabelView();

            return true;
        }

        return false;
    }

    private File searchCurrentDirForFile(File file)
    {
        String filename = file.getName();
        File currentDir = new File(".");
        return new File(currentDir, filename);
    }

    private boolean loadSegmentImage(File currentDirPath)
    {
        try{
            String imageFileName = mdata.getImageFilePath(); 
            File file = new File(imageFileName);
            if(currentDirPath != null && !file.exists()) /* file doens't exist, check the current dir */
            {
                file= new File(currentDirPath, file.getName());
                //System.out.println(file);
            }
            Image image = ImageIO.read(file);
            String segFileName = mdata.getSegmentFilePath();
            file = new File(segFileName);
            if(currentDirPath != null && !file.exists()) /* file doens't exist, check the current dir */
            {
                file= new File(currentDirPath, file.getName());
                //System.out.println(file);
            }
            Vector segments = SegmentLine.parseSegmentFile(file);
            segmentImage.updateSegmentImage(image, new SegmentMap(segments));
            /*segmentImage.setActiveSegmentIndex(0);*/
        }
        catch(Exception e)
        {
            JOptionPane.showMessageDialog(this, 
                        RegionLabelConstants.READ_FILE_ERROR, null, JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
            return false;
        }

         return true;
    }
    
    private boolean loadCorelKeyWords(String filename)
    {
        if(filename == null)
            return false;

        int len = filename.length();
        int index = filename.lastIndexOf("_"); // the file name could be something like 10000_1.lab
        if(index < 0) // not found;
        {
            index = filename.lastIndexOf(".");
        }

        if(index < 0) //invalid file name
            return false;

        String s = filename.substring(0, index);
        Vector v = getCorelKeyWords(Integer.parseInt(s));
        if(v == null)
            return false;

        corelWordList.setListData(v);
        return true;
    }

    private boolean executeSaveFile()
    {
        String tempFile = null;

        if(!isDataChanged)
            return false;

        // keep a copy of the old output file. If save failed ,this will be put back;
       /* File oldOutputFile = null;
        if(outputFile != null)
        {
           oldOutputFile =  new File(outputFile.getPath()); 
        }

        if(isNewFile)
        {
            boolean bret = askWhereToSave();
            if(!bret) return false;  // operation canceled; 
        }*/

        boolean b = RegionLabelData.saveRegionLabelDataAsText(mdata, outputFile.getPath(), true);
        if(!b)
        {
            JOptionPane.showMessageDialog(this, 
                    null, RegionLabelConstants.SAVE_FILE_ERROR, JOptionPane.ERROR_MESSAGE);
            return false;
        }

        setDataChanged(false); // change has saved;
        
        return true;
    }
    
    private boolean executeSaveAsFile()
    {
       // File saveFile = askWhereToSave(RegionLabelConstants.FILE_SAVE_AS_ITEM, "lab", imageFileDirectory, outputFileFilter);
        File saveFile = askWhereToSave(RegionLabelConstants.LABEL_FILE);
        if(saveFile == null) return false;  // operation canceled; 

        boolean b = RegionLabelData.saveRegionLabelDataAsText(mdata, saveFile.getPath(), true);
        if(!b)
        {
            JOptionPane.showMessageDialog(this, 
                    null, RegionLabelConstants.SAVE_FILE_ERROR, JOptionPane.ERROR_MESSAGE);
            return false;
        }

        setOutputFile(saveFile);
        setDataChanged(false); // change has saved;
        this.setTitle(outputFile.getPath());
      
        return true;
    }
    
    /* figure out where to save the file and the result is stored in 'outputFile' */
   /* private File askWhereToSave(String dialogTitle, 
                                   String extension,
                                   File currentDirectory, 
                                   ExtensionFileFilter filter)*/
    private File askWhereToSave(int saveType)
    {
        File saveFile = null;

        JFileChooser fc = new JFileChooser();
        //fc.setDialogType(JFileChooser.SAVE_DIALOG);
        fc.setApproveButtonText("Save"); // awkward because the last line doesn't work
        File defaultFile = null;
        File recentDirectory = null;
        if(saveType == RegionLabelConstants.LABEL_FILE)
        {
            fc.setDialogTitle(RegionLabelConstants.FILE_SAVE_AS_ITEM);
            fc.setFileFilter(outputFileFilter);
            defaultFile = getFileWithDifferentExtension(outputFile.getName(), "lab");
            System.out.println(outputFile);
            System.out.println(outputFile.getParent());
            recentDirectory = new File(outputFile.getParent());
        }
        else if(saveType == RegionLabelConstants.TEMPLATE_FILE)
        {
            fc.setDialogTitle(RegionLabelConstants.FILE_SAVE_TEMPLATE_ITEM);
            fc.setFileFilter(templateFileFilter);
            defaultFile = getFileWithDifferentExtension(outputFile.getName(), "txt");
            recentDirectory = templateFileDirectory;
        }
        else
        {
            System.out.println("Wrong file type!");
            return null;
        }

        while(true) 
        {
            fc.setCurrentDirectory(recentDirectory);
            fc.setSelectedFile(defaultFile);
            int returnVal = fc.showOpenDialog(this);
            if(returnVal == JFileChooser.APPROVE_OPTION) 
            {
                File labelFile = fc.getSelectedFile();
               // check if file exists;
               if(labelFile.exists()) // file exists
               {
                   returnVal = JOptionPane.showConfirmDialog(this,
                         "File exists. Do you want to overwrite it?", "",
                         JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
                   if(returnVal == JOptionPane.YES_OPTION)
                   {
                       saveFile = labelFile;
                       break;                   
                   }
                   else if(returnVal == JOptionPane.NO_OPTION)
                   {
                       recentDirectory = new File(labelFile.getParent());
                   }
                   else
                   {
                       break;
                   }
               }
               else
               {
                   saveFile = labelFile;
                   break;
               }
               
            }
            else
            {
                break;
            }
        }

        return saveFile;
    }
   
    private boolean executeCloseFile(boolean isCleanup)
    {
        boolean ret = true;
        
        if(isDataChanged)
        {
            //need to save change?
            int returnVal = JOptionPane.showConfirmDialog(this,
            "Do you want to save changes?", "",
            JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
        
            if(returnVal == JOptionPane.CANCEL_OPTION)
            {
                return false; // do nothing;
            }

            if(returnVal == JOptionPane.YES_OPTION)
            {
                ret = executeSaveFile();
            }
        }

        //clean up;
        if(ret && isCleanup)
        {
            setOutputFile((File)null);
            setDataChanged(false);
            this.setTitle("");
            wordNetFrame.setVisible(false);
            
            //mdata = null;
            //segmentImage = null;
           // segmentImage.setActiveSegmentIndex(0);
            segmentImage.updateSegmentImage(null, null);
            setBeforeLoadComponentStatus();
        }
       
        return ret; 
    }

    private void executeLoadTemplateFile()
    {
        File templateFile = null;

        JFileChooser fc = new JFileChooser();
        fc.setDialogType(JFileChooser.OPEN_DIALOG);
        fc.setFileFilter(templateFileFilter);
        fc.setDialogTitle(RegionLabelConstants.FILE_LOAD_TEMPLATE_ITEM);
        if(templateFileDirectory.exists())
        {
            fc.setCurrentDirectory(templateFileDirectory);
        }
        else
        {
            fc.setCurrentDirectory(new File("."));
        }

        int returnVal = fc.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) 
        {
            templateFile = fc.getSelectedFile();
            try
            {
                 RegionLabelData d = RegionLabelData.readRegionLabelDataFromText(templateFile.getPath());
                 Vector labels = d.getLabels();
                 
                 // bad code. Remove all items in the combo list and then fill it in with the new template
                 labelListCombo.removeAllItems();

                 labelListCombo.removeActionListener(this);  // avoid select during uploading;
                 int size = newLabelList.size();
                 for(int i=0; i<size; i++) {
                     labelListCombo.addItem(newLabelList.get(i));
                 }
                 if(labels != null)
                 {
                     size = labels.size();
                     for(int i=0; i<size; i++){
                         labelListCombo.addItem(labels.get(i));
                     }
                 }
                 labelListCombo.addActionListener(this);  // restore it;
            }
            catch(Exception e)
            {
                e.printStackTrace();
            }
        }
    }
   
    private boolean executeSaveTemplateFile()
    {

 //       File saveFile = askWhereToSave(RegionLabelConstants.FILE_SAVE_TEMPLATE_ITEM, "txt", templateFileDirectory, templateFileFilter);
        File saveFile = askWhereToSave(RegionLabelConstants.TEMPLATE_FILE);

        if(saveFile == null) return false;
        
        boolean b = RegionLabelData.saveRegionLabelDataAsText(mdata, saveFile.getPath(), true);
        if(!b)
        {
            JOptionPane.showMessageDialog(this, 
                    null, RegionLabelConstants.SAVE_FILE_ERROR, JOptionPane.ERROR_MESSAGE);
            return false;
        }

        return true;
    }


    private void executeQuit()
    {
        if(executeCloseFile(false))
        {
            System.exit(0);
        }
    }

/**************************END OF ROUNTINES FOR OPERATIONS ON 'FILE' MENU**************************/

/**************************START OF ROUNTINES FOR OPERATIONS ON 'VIEW' MENU**************************/
/**************************END OF ROUNTINES FOR OPERATIONS ON 'VIEW' MENU**************************/

/**************************START OF ROUNTINES FOR OPERATIONS ON 'OPTION' MENU**************************/
private void executeChangeTemplateDir()
{
    ;
}

/**************************END OF ROUNTINES FOR OPERATIONS ON 'OPTION' MENU**************************/

/**************************START OF ROUNTINES FOR OPERATIONS ON 'HELP' MENU**************************/
private void executeAbout()
{
JOptionPane.showMessageDialog(this,    "   Region Labeler   \n" + 
                                       "    Version 1.0    \n " + 
                                      "Computer Vision Group\n" +
                                      "University of Arizona");
}
/**************************END OF ROUNTINES FOR OPERATIONS ON 'HELP' MENU**************************/


/************************START OF ROUNTINES FOR OPERATIONS ON 'LabelTextField' COMPONENT*******************/
    private void executeDisplayWordNetPage(WordTextField textField)
    {
        String word = textField.getPointedWord();
        String paddedstr = (word.trim()).replaceAll(" ", "%20"); // replace space between words with something
                                                                 //   reconizable by the http server;
        String urlstr = RegionLabelConstants.WORDNET_URL_STRING+"?o2=&o0=1&o6=1&o1=1&o5=&o4=&o3=&s="+paddedstr;
        try
        {
            URL url = new URL(urlstr);
            wordNetPane.setPage(url);
            wordNetFrame.setVisible(true);
        }
        catch (MalformedURLException e) {
            JOptionPane.showMessageDialog(this, 
                    null, RegionLabelConstants.URL_SYNTAX_ERROR, JOptionPane.ERROR_MESSAGE);
        }
        catch (SecurityException e) {  
            JOptionPane.showMessageDialog(this, 
                    null, RegionLabelConstants.URL_ACCESS_ERROR, JOptionPane.ERROR_MESSAGE);
        }
        catch (IOException e) {  
            JOptionPane.showMessageDialog(this, 
                    null, RegionLabelConstants.URL_READ_ERROR, JOptionPane.ERROR_MESSAGE);
        }
    }

    private void executeHideWordNetPage()
    {
        wordNetFrame.setVisible(false);
    }

    private void executeAssignLabel(WordTextField textField)
    {
       /* if(textField == labelTextField)
        {
            String labelStr = labelTextField.getText();
            if(labelStr != null) // ignore empty string;
            {
                 mdata.setSegmentLabelAt(labelStr, getActiveSegmentIndex());
                 setDataChanged(true);   // mark data as changed;
                
                 updateNewLabelList(labelStr);
                 updateLabelList(labelStr);
                 executeRotateSegment(forwardButton);
            }
            else
            {
                 JOptionPane.showMessageDialog(this, 
                        null, RegionLabelConstants.INVALID_LABEL_ERROR, JOptionPane.ERROR_MESSAGE);
            }
        }*/
        executeRotateSegment(forwardButton);
    }
/************************END OF ROUNTINES FOR OPERATIONS ON 'LabelTextField' COMPONENT*******************/

    private void executeUpdateLabel(JComboBox comboBox)
    {
        if(comboBox == labelListCombo)
        {
            String selectedStr = (String)comboBox.getSelectedItem();
            if(selectedStr != null)
            {
                labelTextField.setText(new String(selectedStr));
                labelTextField.selectAll();
                labelTextField.requestFocusInWindow();
            }
               

     //        System.out.println(selectedStr);
            if(selectedStr != null && !currentLabel.equals(selectedStr))
            {
                updateLabel(selectedStr, false);
                currentLabel = selectedStr;
                setDataChanged(true);
            }
        }
    }
    
    private void executeUpdateLabel(String selectedStr)
    {
        if(selectedStr != null)
        {
            labelTextField.setText(new String(selectedStr));
            labelTextField.selectAll();
            labelTextField.requestFocusInWindow();
        }

        if(selectedStr != null && !currentLabel.equals(selectedStr))
        {
            updateLabel(selectedStr, false);
            currentLabel = selectedStr;
            setDataChanged(true);
        }
    }
    
    private void executeUpdateLabel(SegmentImagePanel segImg)
    {
        // System.out.println("mouse clicked");
        if(segImg == segmentImage)
        {
            updateRegionLabelView(); 
        }
    }

    private void executeClearLabelList()
    {
        labelListCombo.removeAllItems();
        newLabelList.clear();
    }

    private void executeRotateSegment(JButton button)
    {
        //update the label if it's changed;
        String newLabel = labelTextField.getText();
        if(isValidLabel(newLabel))
        {
            /* if trimmed, it will cause seting data changed */
            //updateLabel(newLabel.trim(), true); 
            updateLabel(newLabel, true);
        }
        else
        {
             //throws error;
             JOptionPane.showMessageDialog(this, 
                    RegionLabelConstants.EMPTY_LABEL_ERROR, null, JOptionPane.ERROR_MESSAGE);
             return;
        }

        if(button == forwardButton)
        {
            segmentImage.rotateUpSegment();
        }
        else if(button == backwardButton)
        {
	    segmentImage.rotateDownSegment();
        }

        updateRegionLabelView(); 
    }
  
    private boolean isValidLabel(String label)
    {
         if(label == null)
             return false;

         String s = label.trim();
         if(s == null || s.equals(""))
             return false;

         return true;
    }
 
    private void updateLabel(String newLabel, boolean isListUpdated)
    {
        if(!currentLabel.equals(newLabel)) // label changed;
        {
            mdata.setSegmentLabelAt(newLabel, getActiveSegmentIndex());
            setDataChanged(true);   // mark data as changed;
                
            if(isListUpdated)
            {
                updateNewLabelList(newLabel);
                updateLabelList(newLabel);
            }
        }
    }

    private void updateNewLabelList(String label)
    {
        int size = newLabelList.size();
        int i = 0;

        for(i=0; i<size; i++)
        {
            String itemStr = (String)newLabelList.get(i);
            if(label.equalsIgnoreCase(itemStr))
            {
                break;
            }
        }

        if(i == size)
        {
            newLabelList.add(new String(label));   
        }
    }

    private void updateLabelList(String label)
    {
        int i = 0;

        int itemCount = labelListCombo.getItemCount();
        for(i=0; i<itemCount; i++)
        {
            String itemStr = (String)labelListCombo.getItemAt(i);
            if(label.equalsIgnoreCase(itemStr))
            {
                break;
            }
        }   

         if(i == itemCount)  // new item
         {
             labelListCombo.insertItemAt(new String(label), 0); // put it on the top;
         }
    }
    
    /* Get the Corel key words for a given image*/
    private static Vector getCorelKeyWords(int imageIndex)
    {
        Vector words = null;
        try
        {
            BufferedReader in = new BufferedReader(new FileReader(RegionLabelConstants.COREL_KEYWORDS_FILENAME));
    	    StreamTokenizer tokenizer;
            tokenizer = new StreamTokenizer(in);
            tokenizer.whitespaceChars('+', '+');
            tokenizer.eolIsSignificant(true);
            int token = 0;
            boolean isLocated = false;
            while((token = tokenizer.nextToken()) != StreamTokenizer.TT_EOF)
            {
    		if (token == StreamTokenizer.TT_NUMBER)
                {
                    if(isLocated == true)
                        break;
                    
                    int index = (int)tokenizer.nval;
                   // System.out.println(index);
                    if(index == imageIndex)
                    {
                       words = new Vector();
                       isLocated = true; 
                    }
                }
                else if(token == StreamTokenizer.TT_WORD && isLocated)
                {
 //                   System.out.println((String)tokenizer.sval);
                     words.add((String)tokenizer.sval);
                }
            }
            in.close();
            tokenizer = null;
        }
        catch(IOException e)
        {
            e.printStackTrace();
        }
        finally
        {
            return words;
        }

    }

   /* public static void main(String[] args)
    {
        // Use a label to display the image
        JFrame frame = new RegionLabelFrame();
        frame.pack();
        frame.setVisible(true);
    }*/
}
