import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import java.awt.BorderLayout;

public class SegmentImageLabel extends JLabel
{
    public static final int ALL_SEGMENTS_ON = 1;
    public static final int ACTIVE_SEGMENT_ON = 2;
    public static final int ALL_SEGMENTS_OFF = 2;

    private int numSegments = 0;
    private Vector segments = new Vector();
    private int activeSegmentIndex = 0;
    private int segmentDisplayMode = ALL_SEGMENTS_ON;
    
    SegmentImageLabel()
    {
        addMouseListener(new MyMouseAdapter());
    }
       
    SegmentImageLabel(Vector seg)
    {
        if(seg != null)
        {
            segments = seg;
            numSegments = segments.size();
        }

        addMouseListener(new MyMouseAdapter());
    }
    
    SegmentImageLabel(Image image, Vector seg)
    {
        super(new ImageIcon(image));

        if(seg != null)
        {
            segments = seg;
            numSegments = segments.size();
        }

        addMouseListener(new MyMouseAdapter());
    }

    public void setSegments(Vector seg)
    {
        if(segments != seg)
        {
            segments = seg;
            numSegments = segments.size();
            repaint();
        }
       
    }
    
    public void setSegments(File file)
    {
        setSegments(file.getPath());
    }

    public void setSegments(String fileName)
    {  
        try
        {
            Vector seg = SegmentPoint.parseSegmentFile(fileName);
            setSegments(seg);
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    public int getNumSegments()
    {
        return numSegments;
    } 
    
    public void setNumSegments(int num)
    {
        numSegments = num;
    } 

    public Vector getSegments()
    {
        return segments;
    } 
    
    public int getActiveSegmentIndex()
    {
        return activeSegmentIndex;
    } 
    
    public void setActiveSegmentIndex(int index)
    {
        if(index >= 0 && index < numSegments && 
           index != activeSegmentIndex)
        {
          activeSegmentIndex = index;
          repaint();
        }
    }

    public void rotateUpSegment()
    {
       int index = activeSegmentIndex + 1;
       if(index >= numSegments)
       {
           index = 0;
       }
       
       setActiveSegmentIndex(index); 
    }

    public void rotateDownSegment()
    {
       int index = activeSegmentIndex - 1;
       if(index < 0)
       {
           index = numSegments - 1;
       }
       
       setActiveSegmentIndex(index); 
    }

    public int getSegmentDisplayMode()
    {
        return activeSegmentIndex;
    } 
    
    public void setSegmentDisplayMode(int mode)
    {
        segmentDisplayMode = mode;
    }

    public void paintComponent(Graphics g)
    {
        Graphics2D g2= (Graphics2D)g;
       // super.paintComponent(g2);

        Dimension d = getSize();
	//g.drawImage(icon.getImage(), 0, 0, width, d.height, null);
        ImageIcon icon = (ImageIcon)getIcon();
	if(icon != null)
	 //   g.drawImage(icon.getImage(), 0, 0, d.width, d.height, null);
            g.drawImage(icon.getImage(), 0, 0,  null);

        Color oldColor = g2.getColor();
        if(segmentDisplayMode == ALL_SEGMENTS_ON)
        {
            int size = segments.size();
            for(int i=0; i<size; i++)
            {
                drawSegment(g2, i);   
            }
        }
        else if(segmentDisplayMode == ACTIVE_SEGMENT_ON)
        {
            drawSegment(g2, activeSegmentIndex);   
        }
    }

    private void drawSegment(Graphics2D g2, int index)
    {

        Vector segment = (Vector)segments.elementAt(index);
        if(index == activeSegmentIndex)
        {
            g2.setColor(Color.RED);
            g2.setStroke(new BasicStroke(3.0f));
        }
        else
        {
            g2.setColor(Color.BLUE);
            g2.setStroke(new BasicStroke(1.0f));
        }

        for(int i=0; i<segment.size(); i++)
        {  
            SegmentPoint sPoint = (SegmentPoint)segment.elementAt(i);
            Point pts[] = sPoint.getEndPoints();
            if(sPoint.getConnected())
            {
                g2.drawLine(pts[0].x, pts[0].y, pts[1].x, pts[1].y);
            }
            else
            {
                g2.drawLine(pts[0].x, pts[0].y, pts[0].x, pts[0].y);
                g2.drawLine(pts[1].x, pts[1].y, pts[1].x, pts[1].y);
            }
        }
    }

    public int findSegmentIndex(Point pMouse)
    {
         int x = pMouse.x - getX();
         int y = pMouse.y - getY();

         //System.out.println("x: " + x + " y: " + y + " " + getX() + " " + getY());
         //System.out.println(numSegments);
         for(int i=0; i<numSegments; i++)
         {
             Vector segment = (Vector)segments.elementAt(i);
             for(int j=0; j<segment.size(); j++)
             {
                 SegmentPoint sp = (SegmentPoint)segment.elementAt(j);
                 Point[] ep = sp.getEndPoints();
     //            System.out.println(j + " " + ep[0].x + " " + ep[1].x + " " + ep[0].y);
                 if(ep[0].y == y)
                 {
                     if(x >= ep[0].x && x <= ep[1].x)
                     {
                          return i;
                     }
                 }
             }
         }

         return -1;
    }

    private class MyMouseAdapter extends MouseAdapter
    {
        public void mouseClicked(MouseEvent event)
        {
            int index = findSegmentIndex(event.getPoint());
           // System.out.println("Segment index: " + index);
            setActiveSegmentIndex(index);
        }
    }
    

    public static void main(String[] args)
    {
        Image image = null;
    // Read from a file
        try{
        File file = new File("226033.jpg");
        image = ImageIO.read(file);
        } catch (IOException e){};
        
        // Use a label to display the image
       /* Vector v = new Vector();
        Vector v1 = new Vector();
        Vector v2 = new Vector();
        SegmentPoint sPoint;
        sPoint = new SegmentPoint(new Point(3, 10), new Point(3,20), true);
        v1.add(sPoint);
        sPoint = new SegmentPoint(new Point(10, 5), new Point(60,290), false);
        v1.add(sPoint);
        sPoint = new SegmentPoint(new Point(30, 15), new Point(6,200), false);
        v2.add(sPoint);
        sPoint = new SegmentPoint(new Point(300, 15), new Point(299,200), true);
        v2.add(sPoint);
        v.add(v1);
        v.add(v2);*/

     /*   Vector v = SegmentPoint.parseSegmentFile(args[0]);
        JFrame frame = new JFrame();
        JLabel label = new SegmentImageLabel(image, v);
        frame.getContentPane().add(label, BorderLayout.CENTER);
        frame.pack();
        frame.setVisible(true);*/
    }
}
