import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.*;
import java.util.*;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JFrame;
import java.awt.BorderLayout;

public class SegmentImagePanel extends JLabel
{
    public static final int ALL_SEGMENTS_ON = 1;
    public static final int ACTIVE_SEGMENT_ON = 2;
    public static final int ALL_SEGMENTS_OFF = 3;
    public static final int IMAGE_AND_SEGMENTS_OFF = 4;

    public static final String IMAGE_CLICKED_STR="IMAGE_CLICKED";

    private Image image = null;
    private SegmentMap map = null;
    private int activeSegmentIndex = 0;
    private int segmentDisplayMode = ALL_SEGMENTS_ON;
    
    SegmentImagePanel()
    {
        this(null, null);
    }
    
    SegmentImagePanel(Image aImage)
    {
        this(aImage, null);
    }
       
    SegmentImagePanel(SegmentMap aMap)
    {
        this(null, aMap);
    }
    
    SegmentImagePanel(Image aImage, SegmentMap aMap)
    {
        map = aMap;
        image = aImage;

        addMouseListener(new MyMouseAdapter(this));
    }
    
    public void setMouseClickedNotified(boolean b)
    {
        if(b)
	    enableEvents(AWTEvent.ACTION_EVENT_MASK); // enble the action event;
        else
	    disableEvents(AWTEvent.ACTION_EVENT_MASK); // enble the action event;
    }
 /*   public void setSegments(Vector aSegments)
    {
        if(aSegments != null && segments != aSegments)
        {
            segments = aSegments;
            repaint();
        }
       
    }
    
    public void setSegments(File file)
    {
        setSegments(file.getPath());
    }

    public void setSegments(String fileName)
    {  
        try
        {
            Vector seg = SegmentPoint.parseSegmentFile(fileName);
            setSegments(seg);
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }*/

    public void updateSegmentImage(Image aImage, SegmentMap aMap)
    {
        map = aMap;
        image = aImage;
        activeSegmentIndex = 0; /* this has to be reset 0 otherwise it would cause problem in some cases*/
        repaint();
    }

    public int getNumSegments()
    {
        return map.getNumSegments();
    } 
    
    public SegmentMap getMap()
    {
        return map;
    } 
    
    public int getActiveSegmentIndex()
    {
        return activeSegmentIndex;
    } 
    
    public void setActiveSegmentIndex(int index)
    {
        if(index >= 0 && index < getNumSegments() && 
           index != activeSegmentIndex)
        {
          activeSegmentIndex = index;
          repaint();
        }
    }

    public void rotateUpSegment()
    {
       int index = activeSegmentIndex + 1;
       int numSegments = getNumSegments();
       if(index >= numSegments)
       {
           index = 0;
       }
       
       setActiveSegmentIndex(index); 
    }

    public void rotateDownSegment()
    {
       int index = activeSegmentIndex - 1;
       int numSegments = getNumSegments();
       if(index < 0)
       {
           index = numSegments - 1;
       }
       
       setActiveSegmentIndex(index); 
    }

    public int getSegmentDisplayMode()
    {
        return activeSegmentIndex;
    } 
    
    public void setSegmentDisplayMode(int mode)
    {
        segmentDisplayMode = mode;
    }

    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if(image == null || map == null)
            return;

        int width = ((BufferedImage)image).getWidth(); 
        int height = ((BufferedImage)image).getHeight(); 
        g.drawImage(image, 0, 0,  null);
       // g.drawImage(image, 0, 0,  2 * width, 2 * height, null);
           
        if(segmentDisplayMode == ALL_SEGMENTS_OFF)
            return;
        
        Graphics2D g2= (Graphics2D)g;
        Color oldColor = g2.getColor();
        drawSegment(g2);
        g2.setColor(oldColor); // restore the origional color;
    }

    private void drawSegment(Graphics2D g2)
    {
        int width = map.getWidth();
        int height = map.getHeight();
        int[][] segMap = map.getSegmentMap();
        boolean[][] bMap = map.getBoundaryMap();

        for(int i=0; i<height; i++)
        {
            for(int j=0; j<width; j++)
            {
                if(!bMap[i][j]) continue;
                if(segMap[i][j] == activeSegmentIndex)
                {
                    g2.setColor(Color.RED);
                    g2.setStroke(new BasicStroke(3.0f));
                }
                else
                {
                    if(segmentDisplayMode == ACTIVE_SEGMENT_ON) continue;
 
                    g2.setColor(Color.BLUE);
                    g2.setStroke(new BasicStroke(1.0f));
                }
        
                g2.drawLine(j, i, j, i); // draw a point;
      //          System.out.println(j + " " + i);
            }
        }
    }

    public int findSegmentIndex(Point pMouse)
    {
         int x = pMouse.x - getX();
         int y = pMouse.y - getY();
/*
         //System.out.println("x: " + x + " y: " + y + " " + getX() + " " + getY());
         int numSegments = segments.size();
         //System.out.println(numSegments);
         for(int i=0; i<numSegments; i++)
         {
             Vector segment = (Vector)segments.elementAt(i);
             for(int j=0; j<segment.size(); j++)
             {
                 SegmentPoint sp = (SegmentPoint)segment.elementAt(j);
                 Point[] ep = sp.getEndPoints();
     //            System.out.println(j + " " + ep[0].x + " " + ep[1].x + " " + ep[0].y);
                 if(ep[0].y == y)
                 {
                     if(x >= ep[0].x && x <= ep[1].x)
                     {
                          return i;
                     }
                 }
             }
         }

         return -1;*/

        // System.out.println("x: " + x + " y: " + y + " " + getX() + " " + getY());
         if(map == null)
             return -1;

         int[][] segMap = map.getSegmentMap();
         if(x < map.getWidth() && y < map.getHeight())
         {
             return segMap[y][x];
         }
        
         return -1;
    }

    private class MyMouseAdapter extends MouseAdapter
    {
        private SegmentImagePanel segPanel = null;

        MyMouseAdapter(SegmentImagePanel sp)
        {
            segPanel = sp;
        }

        public void mouseClicked(MouseEvent event)
        {
            int index = findSegmentIndex(event.getPoint());
           // System.out.println("Segment index: " + index);
            setActiveSegmentIndex(index);
	    if(index >= 0)
            {
                ActionEvent ev = new ActionEvent(segPanel, ActionEvent.ACTION_PERFORMED, IMAGE_CLICKED_STR);
                dispatchEvent(ev);
            }
        }
    }
    
   /**************Respond to events*******************************/
    public void addActionListener(ActionListener l) {
		this.listenerList.add(ActionListener.class, l);
	}

	public void removeActionListener(ActionListener l) {
		this.listenerList.remove(ActionListener.class, l);
	}

	protected void processEvent(AWTEvent ev) {
		super.processEvent(ev);

		if(ev instanceof ActionEvent) { // handle item event;
			processActionEvent((ActionEvent)ev);
		}
	}

	protected void processActionEvent(ActionEvent ev) {
		EventListener[] eList = this.listenerList.getListeners(ActionListener.class);
		for(int i=0; i<eList.length; i++) {
			((ActionListener)eList[i]).actionPerformed(ev);
		}
	}
/*****************************************************************/

    public static void main(String[] args)
    {
        Image image = null;
        Vector v = null;
    // Read from a file
        try
        {
            File file = new File("226033.jpg");
            image = ImageIO.read(file);
            v = SegmentLine.parseSegmentFile(args[0]);
        } catch (Exception e){};
        
        // Use a label to display the image
       /* Vector v = new Vector();
        Vector v1 = new Vector();
        Vector v2 = new Vector();
        SegmentPoint sPoint;
        sPoint = new SegmentPoint(new Point(3, 10), new Point(3,20), true);
        v1.add(sPoint);
        sPoint = new SegmentPoint(new Point(10, 5), new Point(60,290), false);
        v1.add(sPoint);
        sPoint = new SegmentPoint(new Point(30, 15), new Point(6,200), false);
        v2.add(sPoint);
        sPoint = new SegmentPoint(new Point(300, 15), new Point(299,200), true);
        v2.add(sPoint);
        v.add(v1);
        v.add(v2);*/

        JFrame frame = new JFrame();
        SegmentMap map = new SegmentMap(v);
        SegmentImagePanel simage = new SegmentImagePanel(image, map);
        simage.setSegmentDisplayMode(ACTIVE_SEGMENT_ON);
        frame.getContentPane().add(simage, BorderLayout.CENTER);
        frame.setPreferredSize(new Dimension(400, 400));
        frame.pack();
        frame.setVisible(true);
    }
}
