import java.io.Serializable;
import java.awt.Point;
import java.util.Vector;
import java.lang.String;
import java.io.IOException;
import java.io.FileInputStream;

/* This class includes two maps to represent an image segmentation. The first one indicates
which segment a pixel belongs to and the second one indicates whether that pixel is on the
boundary of some segment */
public class SegmentMap implements Serializable
{
        //
    private int width;
    private int height;
    private int[][] segMap = null;
    private boolean[][] bMap = null;
    private int numSegments = 0;
 
    SegmentMap(int aWidth, int aHeight, int aNumSegments)
    {
        setMap(aWidth, aHeight, aNumSegments);
    }
    

    SegmentMap(Vector segmentLineV)
    {
        int numS = -1;
        int w = -1;
        int h = -1;

        int size = segmentLineV.size();
        for(int i=0; i<size; i++)
        {
            SegmentLine sl = (SegmentLine)segmentLineV.get(i);
            if(sl.segmentIndex > numS)
                numS = sl.segmentIndex;
            if(sl.x1 > w)
                w = sl.x1;
            if(sl.x2 > w)
                w = sl.x2;
            if(sl.y > h)
                h = sl.y;
        }

        System.out.println(numS + " " + w + " " + h);
        setMap(w+1, h+1, numS+1);  //

        for(int i=0; i<size; i++)
        {
            SegmentLine sl = (SegmentLine)segmentLineV.get(i);
            for(int j=sl.x1; j<=sl.x2; j++)
            {
                segMap[sl.y][j] = sl.segmentIndex;
            }
        } 

        int segment = 0;
        for(int i=0; i<height; i++) 
        {
            for(int j=0; j<width; j++)
            {
                if(i-1 < 0)
                {
                     bMap[i][j] = true;
                     continue;
                }
                if(i+1 == height)
                {
                     bMap[i][j] = true;
                     continue;
                }
                if(j-1 < 0)
                {
                     bMap[i][j] = true;
                     continue;
                }
                if(j+1 == width)
                {
                     bMap[i][j] = true;
                     continue;
                }
                
                segment = segMap[i][j];
                if(segMap[i-1][j] != segment || segMap[i+1][j] != segment ||
                   segMap[i][j-1] != segment || segMap[i][j+1] != segment)
                {
                     bMap[i][j] = true;
                }
                else
                {
                     bMap[i][j] = false;
                }
            }
        }
    }

    public int getNumSegments()
    {
        return numSegments;
    }

    public int getWidth()
    {
        return width;
    }

    public int getHeight()
    {
        return height;
    }

    public int[][] getSegmentMap()
    {
        return segMap;
    }

    public boolean[][] getBoundaryMap()
    {
        return bMap;
    }

    private void setMap(int aWidth, int aHeight, int aNumSegments)
    {
        width = aWidth;
        height = aHeight;
        numSegments = aNumSegments;
 
        segMap = new int[height][width];
        bMap = new boolean[height][width];
    }
    
   public static void main(String[] args)
    {
        Vector v = null;
    // Read from a file
        try
        {
            v = SegmentLine.parseSegmentFile(args[0]);
        } catch (Exception e){e.printStackTrace();};
     
        SegmentMap map = new SegmentMap(v);
        int width = map.getWidth();   
        int height = map.getHeight();   
        boolean[][] bMap = map.getBoundaryMap();
        for(int i=0; i<height; i++)
        {
           for( int j=0;j<width; j++)
           {
               int b = bMap[i][j] == true? 1: 0;
               System.out.print(b+ " ");
           }
           System.out.println("");
        }
    }
}
