import java.io.Serializable;
import java.awt.Point;
import java.util.Vector;
import java.lang.String;
import java.io.IOException;
import java.io.FileInputStream;

public class SegmentPoint implements Serializable
{
        //
    private Point[] endPoints = new Point[2]; 
    private boolean isConnected = false;
    
    SegmentPoint()
    {
    }
    SegmentPoint(Point p1, Point p2)
    {
        this(p1, p2, false);
    }
    
    SegmentPoint(Point[] p)
    {
        this(p[0], p[1]);
    }

    SegmentPoint(Point p1, Point p2, boolean b)
    {
         endPoints[0] =new Point(p1);
         endPoints[1] = new Point(p2);
         isConnected = b;
    }


    public Point[] getEndPoints()
    {
        return endPoints;
    }
    
    public void setEndPoints(Point[] p)
    {
         endPoints[0] = new Point(p[0]);
         endPoints[1] = new Point(p[1]);
    }
     
    public boolean getConnected()
    {
        return isConnected;
    } 

    public void setConnected(boolean b)
    {
        isConnected = b;
    }

    public String toString()
    {
        // to do;

        return endPoints[0].x + " " + endPoints[0].y + " " + endPoints[1].x + " " + endPoints[1].y + " " + isConnected;
    }

    /* this function assume that the y coordinates of the four points are the same */
    public boolean intersect(SegmentPoint sp)
    {
        int x1 = endPoints[0].x;
        int x2 = endPoints[1].x;
        Point[] p = sp.getEndPoints();
        int x3 = p[0].x;
        int x4 = p[1].x;

        //return !((x3-x1)*(x3-x2) > 0 && (x4-x1)*(x4-x2) > 0);
        return (x3-x1)*(x3-x2) <= 0 || (x4-x1) * (x4-x2) <= 0 || (x1-x3) * (x1-x4) <=0 || (x2-x3) * (x2-x4) <= 0;
    }


    private static void adjustSegmentPointStatus(Vector segment)
    {
        if(segment == null)
        {
            return;
        }

        SegmentPoint prevSegPoint = null;
        SegmentPoint nextSegPoint = null;
        SegmentPoint currentSegPoint = null;
        int size = segment.size();
        for(int i=0; i<size; i++)
        {
            //System.out.println("segment: " + i); 
            currentSegPoint = (SegmentPoint)segment.elementAt(i);
            Point[] cp = currentSegPoint.getEndPoints();
           // System.out.println(cp); 
             //search upward
            boolean upperBound = true;
            for(int j=i-1; j>=0; j--)
            {
                prevSegPoint = (SegmentPoint)segment.elementAt(j);
                Point[] pp = prevSegPoint.getEndPoints();
                if(pp[0].y == cp[0].y - 1)
                {
                    if(currentSegPoint.intersect(prevSegPoint))
                    {
         //               System.out.println("---" + cp[0].x + " " + cp[1].x + " " + pp[0].x + " " + pp[1].x);
                        upperBound = false;
                    }
                }
            }

            //search downward;
            boolean lowerBound = true;
            for(int j=i+1; j<size; j++)
            {
                nextSegPoint = (SegmentPoint)segment.elementAt(j);
                Point[] np = nextSegPoint.getEndPoints();
                if(np[0].y == cp[0].y + 1)
                {
                    if(currentSegPoint.intersect(nextSegPoint))
                    {
                        lowerBound = false;
                        break;
                    }
                }
            }

            if(upperBound || lowerBound)
            {
                currentSegPoint.setConnected(true);
               // System.out.println("set connected");
            } 
        }

    }

    public static Vector parseSegmentFile(String fileName) throws Exception
    {
        
        TextReader input;
        try 
        {
            input = new TextReader(new FileInputStream(fileName));
        } 
        catch (IOException e)
        {
          throw new RuntimeException(e.toString());
        }

        boolean isData = false;
        boolean isNumOfSegments = false;
        int count = 0;
        Vector segments = null;
        int segIndex=0;
        int ypos = 0;
        int xpos1 = 0;
        int xpos2 = 0;
        SegmentPoint spoint = null;
        int numSegments = 0;
        while (input.ready())
        {
            if(!isData) 
            {
                if(isNumOfSegments)
                {
                    numSegments = input.readInt();
                    segments = new Vector();
                    for(int i=0; i<numSegments; i++)
                    {
                       segments.add(new Vector());
                        
                    }
                    isNumOfSegments = false;
                    System.out.println("# of segments: " + numSegments);
                }
                else 
                {
                    String s = input.readWord();
                    if(s.compareTo("data") == 0)
                    {
                        isData = true;
                    }
                    else if(s.compareTo("segments") == 0)
                    {
                        isNumOfSegments = true;
                    }
                } 
            } 
            else
            {
               int id = count % 4;
               if( id == 0)
               {
                   segIndex = input.readInt();
               }
               else if(id == 1)
               {
                   ypos = input.readInt();
               }
               else if(id == 2)
               {
                   xpos1 = input.readInt();
               }
               else if(id == 3)
               {
                   xpos2 = input.readInt();
               }
               
               count++; 

               if(count % 4 == 0) 
               {
 //                  System.out.println(segIndex+" " + ypos + " " + xpos1 + " " + xpos2);
                   spoint = new SegmentPoint(new Point(xpos1, ypos), new Point(xpos2, ypos));
                   Vector v = (Vector)segments.get(segIndex);
                   v.add(spoint);
               } 
            }
        }

        if(segments == null)
        {
           throw new Exception("Invalid segment file!");
        }
 
        for(int i=0; i<segments.size(); i++)
        {
            Vector segment = (Vector)segments.elementAt(i);
            adjustSegmentPointStatus(segment);
        }

        return segments;
    }

        
    public static void main(String[] args)
    {
       try{
           Vector v = parseSegmentFile(args[0]);
           for(int i=0; i<v.size(); i++) {
               System.out.println("-----" + i + "-----");
               Vector pv = (Vector)v.get(i);
               for(int j=0; j<pv.size(); j++)
               {
                   System.out.println((SegmentPoint)pv.get(j));
               }
           }
       }
       catch(Exception e){
           ;
       }
    }
}
