import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.JTextField;
import javax.swing.JFrame;

/* This class is an extended class of JTextField which can fire an event
    whether a RIGHT BUTTON MOUSE click happens on a word of the text or not. */
public class WordTextField extends JTextField
{
    public static final String MOUSE_MOVE_ON_WORD="MOUSE_ON_WORD";
    public static final String MOUSE_MOVE_OFF_WORD="MOUSE_OFF_WORD";
    String pointedWord = ""; // word pointed by mouse
 
    WordTextField()
    {
        super();
        addMouseListener(new MyMouseAdapter(this));
 //       addMouseMotionListener(new MyMouseMotionAdapter(this));
    }
    
    WordTextField(int columns)
    {
        super(columns);
        addMouseListener(new MyMouseAdapter(this));
   //     addMouseMotionListener(new MyMouseMotionAdapter(this));
    }

    WordTextField(String text, int columns)
    {
        super(text, columns);
        addMouseListener(new MyMouseAdapter(this));
     //   addMouseMotionListener(new MyMouseMotionAdapter(this));
    }

    public String getPointedWord()
    {
        return pointedWord;
    }

    public void setPointedWord(String word)
    {
        pointedWord = word; 
    }
 /*   private void createDisplayFrame()
    {
        displayFrame = new JFrame();
     //   displayFrame.setUndecorated(true);
        displayArea = new JTextPane();
        JScrollPane js = new JScrollPane();
        js.getViewport().add(displayArea);
        displayFrame.getContentPane().add(displayArea);
        displayFrame.setPreferredSize(new Dimension(300, 150));
        displayFrame.pack();
        displayFrame.setVisible(false);
    }*/
   
    protected String identifyWord(int mouseX, int mouseY)
    {
        mouseX += getScrollOffset(); // might have scrolled;

        Font textFont = getFont();
        FontMetrics fm = getFontMetrics(textFont);
        int textHeight = fm.getAscent() + fm.getDescent();
       // System.out.println(getHeight() + " " + textHeight + " " + mouseX + " " + mouseY);
        mouseY -= ((getHeight() - textHeight) / 2);
        if(mouseY < 0 || mouseY > textHeight)
            return null;

        String text = getText();
        int length = text.length();
        int xpos = 0;
        int curIndex = 0;
        for(curIndex=0; curIndex<length; curIndex++)
        {
            int charwidth = fm.charWidth(text.charAt(curIndex));
            xpos += charwidth;
            if(mouseX < xpos)
                 break;
        }

        if(curIndex == length || !Character.isLetter(text.charAt(curIndex)))
            return null;

        int startOfWord = curIndex;
        for(startOfWord=curIndex-1; startOfWord>=0; startOfWord--)
        {
            if(!Character.isLetter(text.charAt(startOfWord)))
            {
                 break;
            }
        }
        startOfWord++;
        
        int endOfWord = curIndex;
        for(endOfWord=curIndex+1; endOfWord<length; endOfWord++)
        {
            if(!Character.isLetter(text.charAt(endOfWord)))
            {
                break;
            }
        }

        return text.substring(startOfWord, endOfWord);
    }
 
    private class MyMouseAdapter extends MouseAdapter
    {
        private WordTextField wordText;

        MyMouseAdapter(WordTextField aWordText)
        {
            wordText = aWordText;
        }

        public void mouseClicked(MouseEvent e)
        {
            if(e.getButton() != MouseEvent.BUTTON3)  // only handle right button event
                return;

            int x = e.getX();
            int y = e.getY();
            String word = null;
            word = getSelectedText();  // check if something is selected;
            if(word == null || word.trim().length() == 0)  // if not, then identify a word where the mouse points to
            {
                word = identifyWord(x, y);
            }
            System.out.println(word);

            if(word !=null)
            {
              //  System.out.println(word);
                setPointedWord(word);
                ActionEvent ev = new ActionEvent(wordText, ActionEvent.ACTION_PERFORMED, MOUSE_MOVE_ON_WORD);
                dispatchEvent(ev);
            }
        }
        
       /* public void mouseExited(MouseEvent e)
        {
            ActionEvent ev = new ActionEvent(wordText, ActionEvent.ACTION_PERFORMED, MOUSE_MOVE_OFF_WORD);
            dispatchEvent(ev);
        }*/
    }

   /* private class MyMouseMotionAdapter extends MouseMotionAdapter
    {
        private WordTextField wordText;

        MyMouseMotionAdapter(WordTextField aWordText)
        {
            wordText = aWordText;
        }

        public void mouseMoved(MouseEvent e)
        {
            int x = e.getX();
            int y = e.getY();
            String word = identifyWord(x, y);
            if(word !=null)
            {
              //  System.out.println(word);
                setPointedWord(word);
                ActionEvent ev = new ActionEvent(wordText, ActionEvent.ACTION_PERFORMED, MOUSE_MOVE_ON_WORD);
                dispatchEvent(ev);
            }
            else
            {
                ActionEvent ev = new ActionEvent(wordText, ActionEvent.ACTION_PERFORMED, MOUSE_MOVE_OFF_WORD);
                dispatchEvent(ev);
            } 
        }

        public void mouseExited(MouseEvent e)
        {
            ActionEvent ev = new ActionEvent(wordText, ActionEvent.ACTION_PERFORMED, MOUSE_MOVE_OFF_WORD);
            dispatchEvent(ev);
        }
    }*/
    
    /* If key is typed, notify listener that the text is changed */
  /*  private class MyKeyAdapter extends KeyAdapter
    {
        private WordTextField wordText;

        MyKeyAdapter(WordTextField aWordText)
        {
            wordText = aWordText;
        }

        public void keyTyped(KeyEvent e)
        {
        }
    }*/
   
    public void setMouseOnWordNotified(boolean b)
    {
        if(b)
	    enableEvents(AWTEvent.ACTION_EVENT_MASK); // enble the action event;
        else
	    disableEvents(AWTEvent.ACTION_EVENT_MASK); // enble the action event;
    }

   /**************Respond to events*******************************/
    public void addActionListener(ActionListener l) {
		this.listenerList.add(ActionListener.class, l);
	}

	public void removeActionListener(ActionListener l) {
		this.listenerList.remove(ActionListener.class, l);
	}

	protected void processEvent(AWTEvent ev) {
		super.processEvent(ev);

		if(ev instanceof ActionEvent) { // handle item event;
			processActionEvent((ActionEvent)ev);
		}
	}

	protected void processActionEvent(ActionEvent ev) {
		EventListener[] eList = this.listenerList.getListeners(ActionListener.class);
		for(int i=0; i<eList.length; i++) {
			((ActionListener)eList[i]).actionPerformed(ev);
		}
	}
/*****************************************************************/
   
    /* modified based on the source code from 
       http://www.javacoffeebreak.com/books/extracts/javanotesv3/c10/s4.html/ */ 
   /* public static String getHttpTextContent(String httpString)
    {
         try
         {
             URL url = new URL(httpString);
             Object content = url.getContent();
             if (content instanceof String)
             {
                 return (String)content;
             }
             else if (content instanceof InputStream || 
                      content instanceof Reader)
             {
                
                 BufferedReader in = null;
                 if(content instanceof InputStream)
                 {
                      in =  new BufferedReader(new InputStreamReader((InputStream)content));
                 }
                 else if (content instanceof BufferedReader)
                 {
                      in = (BufferedReader)content;
                 }
                 else 
                 {
                      in = new BufferedReader( (Reader)content );
                 }
                 char[] data = new char[10000]; // The characters read.
                 int index = 0;                 // Number of chars read.
                 while (true) {
                    // Read up to 10000 chars and store them in the array.
                    if (index == 10000)
                       break;
                     int ch = in.read();
                     if (ch == -1)
                        break;
                     data[index] = (char)ch;
                     index++;
                  }
                  in.close();
                  if (index == 0)
                  {
                       return("Error: couldn't read any data.");
                  }
                  else
                  {
                       return new String(data,0,index);
                  }
             }
             else {
                   return new String("Error: Not pure plain text.");
             }
         }
         catch (MalformedURLException e) {
             return new String("Error:improper syntax given for the URL to be loaded.");
         }
         catch (SecurityException e) {  
             return new String("Error:couldn't access that URL.");
         }
         catch (IOException e) {  
             return new String("Error:problem reading data from that URL:\n" + e.toString());
      }

    }*/

    public static void main(String[] args)
    {
        WordTextField w = new WordTextField(40);
        JFrame frame = new JFrame();
        frame.getContentPane().add(w);
        frame.setPreferredSize(new Dimension(150, 50));
        frame.pack();
        frame.setVisible(true);
    }
}
